/*
  Arduino Mega Server for ESP8266 SD
  version 0.15, English edition
  2016, Hi-Lab.ru
  
  License: Free, without any warranties
  Home:    http://hi-lab.ru/arduino-mega-server (russian)
           http://hi-lab.ru/english/arduino-mega-server (english)
  Email:   info@hi-lab.ru
  
  IDE: Arduino 1.6.5 r2
  ESP8266 drivers: 2.0.0
  Board: ESP8266 with SD card

  Settings:
  ---------
  In Sketch (tab Wi-Fi): SSID, PASSWORD of your Wi-Fi network
  In browser: IP address 192.168.1.43
*/

// modules
#define SD_FEATURE
#define SD_INFO_FEATURE
//#define SD_FILES_FEATURE
#define RTC_FEATURE
#define NTP_FEATURE
#define HTTP_FEATURE
//#define FTP_FEATURE
#define SEND_FEATURE
//#define MAJORDOMO_FEATURE
//#define LAURENT_FEATURE
//#define CONTACTS_FEATURE
//#define PIRS_FEATURE
//#define TEMP_FEATURE
//#define LEDS_FEATURE
//#define KEYS_FEATURE


// #define ELECTRO_FEATURE

 #define NRF24_FEATURE

 #define ROBOTDYN_FEATURE
 //===================================
 #define WUNDERGR_FEATURE
//==================================
 

// debug
//#define ELECTRO_DEBUG
//#define EVENTS_CONTACTS
//#define EVENTS_PIRS
//#define TEMP_DEBUG
#define SERIAL_PRINT

#include <SPI.h>
#include <ESP8266WiFi.h>
#include <DNSServer.h>
#include <SD.h>
//#include <Time.h>

#include <TimeLib.h>


/*
#include <Wire.h>


 #include <LiquidCrystal_I2C.h>       // Подключаем библиотеку

  LiquidCrystal_I2C lcd(0x27,20,4);    // Назначаем адрес и размер экрана..20,4..или 16,2
*/
char const SELF_NAME[] = "WeMos_SD_6_wg";
byte SELF_IP[] = {192, 168, 2, 45};

// Authorization
#define AUTH_OFF 0
#define AUTH_ON  1
byte authMode = AUTH_OFF;
// online encode: base64encode.org
String AUTH_HASH = "Authorization: Basic YWRtaW46YW1z"; // admin:ams

// time provider
#define TIME_NONE    0
#define TIME_NETWORK 1
#define TIME_RTC     2
byte timeProvider = TIME_NETWORK;





// duration
time_t prevEventElectroCycle;
time_t prevEventMegaLive;
time_t prevEventRequest;



#ifdef NRF24_FEATURE




#endif // NRF24_FEATURE


// hardware structure


typedef struct Hardware {
  byte platform  = 0;
  byte ethernet  = 0;
  byte sdType    = 0;
  byte sdFs      = 0;
  int  sdVolume  = 0;
  byte nrf24L01  = 0;
  byte nooMt1132 = 0;
  byte nooMr1132 = 0;
  byte rtc       = 0;
} hardware;

hardware hard;


// esp partner
#define MEGA_OFF 0
#define MEGA_ON  1
byte mega = MEGA_OFF;

// modules
#define MODUL_DISABLE       0
#define MODUL_ENABLE        1
#define MODUL_NOT_COMPILLED 2
byte modulRtc      = MODUL_NOT_COMPILLED;
byte modulNtp      = MODUL_NOT_COMPILLED;
byte modulSend     = MODUL_NOT_COMPILLED;
byte modulMajor    = MODUL_NOT_COMPILLED;
byte modulLaurent  = MODUL_NOT_COMPILLED;
byte modulPirs     = MODUL_NOT_COMPILLED;
byte modulContacts = MODUL_NOT_COMPILLED;
byte modulTemp     = MODUL_NOT_COMPILLED;
byte modulLeds     = MODUL_NOT_COMPILLED; 
byte modulKeys     = MODUL_NOT_COMPILLED;
byte modulFtp      = MODUL_NOT_COMPILLED;

byte modulEthernet = MODUL_NOT_COMPILLED;
byte modulSd       = MODUL_NOT_COMPILLED;
byte modulServer   = MODUL_NOT_COMPILLED;
byte modulPing     = MODUL_NOT_COMPILLED;
byte modulUpload   = MODUL_NOT_COMPILLED;
byte modulElectro  = MODUL_NOT_COMPILLED;
byte modulNoo      = MODUL_NOT_COMPILLED;
byte modulMr1132   = MODUL_NOT_COMPILLED;
byte modulNrf24    = MODUL_NOT_COMPILLED;

byte modulRobotdyn = MODUL_NOT_COMPILLED;
//=============================================
byte modulTwundergr =MODUL_NOT_COMPILLED;

//===============================================
//==============================================================
/*
// electro
boolean buttonElectro = 0;

// oscill
boolean oscill = false;

// cyclos
unsigned long cyclos = 0;
*/
//==================================================================
/*
// timers
boolean cycle1s  = false;
boolean cycle4s  = false;
boolean cycle20s = false;
boolean cycle30s = false;
boolean cycle1m  = false;
boolean cycle3m  = false;
boolean cycle5m  = false;
*/


// timers

unsigned long timeSec; // time in seconds
unsigned long timer1s;  boolean cycle1s  = false;
unsigned long timer4s;  boolean cycle4s  = false;
unsigned long timer20s; boolean cycle20s = false;
unsigned long timer30s; boolean cycle30s = false;
unsigned long timer1m;  boolean cycle1m  = false;
unsigned long timer3m;  boolean cycle3m  = false;
unsigned long timer5m;  boolean cycle5m  = false;
int startSendTime = 10;

// strings
char buf[200];

// for sample Ajax
boolean LED_state[4] = {0};

#ifdef LEDS_FEATURE
  // LED modes
  #define LED_EMPTY 0
  #define LED_PIR 1
  byte modeLed = LED_EMPTY;
#endif

//========================================================
char markerWork[] = "";
//==========================================================

/* setup
------------------------------------- */

void setup() {
  Serial.begin(115200);
  Serial.print(F("\nAMS for "));
  Serial.print(SELF_NAME);
  Serial.println(" started...");
  hardwareInit();
  timersInit();
  wifiInit();
  //spiffsInit();
  sdCardInit();
  #ifdef SD_INFO_FEATURE
    sdInfoInit();
  #endif
  #ifdef SD_FILES_FEATURE
    sdFilesInit();
  #endif
  randomInit();
  eepromInit();
  ntpInit();
  rtcInit();
  #ifdef HTTP_FEATURE
    httpInit();
  #endif
  #ifdef FTP_FEATURE
    ftpInit();
  #endif
  serverInit();
  #ifdef MAJORDOMO_FEATURE
    majordomoInit();
  #endif
  #ifdef LAURENT_FEATURE
    laurentInit();
  #endif
  #ifdef TEMP_FEATURE
    tempInit();
  #endif

  //============================================
   #ifdef ELECTRO_FEATURE
   electroInit();
  #endif
//==============================================

  #ifdef CONTACTS_FEATURE
    contactsInit();
  #endif
  #ifdef PIRS_FEATURE
    pirsInit();
  #endif
  #ifdef LEDS_FEATURE
    ledsInit();
  #endif
  #ifdef KEYS_FEATURE
    keysInit();
  #endif


     //================================================

  
#ifdef NRF24_FEATURE
    nrf24Init();
  #endif


  //================================================

  #ifdef ROBOTDYN_FEATURE
    robotdynInit();
  #endif

  #ifdef WUNDERGR_FEATURE
    twundergrInit();
  #endif

  
  Serial.print(F("GLOBAL Init DONE ("));
  Serial.print(millis() / 1000);
  Serial.println(F("s)\n"));
  Serial.println(F("AMS WORK"));
  timeStamp(); printFreeMem("");
} // setup






/* setup
------------------------------------- */

void loop() {
  timersWorks();
  rtcWorks();
  //clockWork();
  
  #ifdef HTTP_FEATURE
    httpWork();
  #endif
  #ifdef FTP_FEATURE
    ftpWork();
  #endif
  
  serverWorks();
  
  #ifdef TEMP_FEATURE
    if (cycle5m) {tempWorks();}
  #endif

  #ifdef ROBOTDYN_FEATURE
    robotdynWorks();
  #endif

//===================================================================


    #ifdef ELECTRO_FEATURE
      if (modulElectro == 1) {
       if (cycle20s || (timeSec < startSendTime)) {
          electroWorks();
          freqWorks();
        }
        // network oscilloscope
        if (cycle1s && oscill) {
          freqWorks();
        }
      }
    #endif
   

 //==========================================================
  
  #ifdef CONTACTS_FEATURE
    contactsWorks();
  #endif
  #ifdef PIRS_FEATURE
    pirsWorks();
  #endif
  #ifdef KEYS_FEATURE
    keysWorks();
  #endif
  #ifdef LAURENT_FEATURE
    if (cycle5m) {
      sprintf(buf, "$KE");
      sendLaurentRequest();
    }
  #endif


  //=============================================
   #ifdef MAJORDOMO_FEATURE
      majordomoMegaLive();
    #endif

//==============================================
  #ifdef WUNDERGR_FEATURE
    twundergrWorks();
  #endif

  //======================================
   #ifdef NRF24_FEATURE
   
      nrf24Works();

     
    #endif

    //========================================    ????????



  if (cycle20s) {
    //
  }

  if (cycle1m) {
    timeStamp(); printFreeMem("");
  }

  // end loop works
  cyclosInSecWork();
  cyclosDelayWork();
  eraseCyclos();
} // loop

