/*
  Modul SD
  part of Arduino Mega Server project
*/

#define SD_CHIP_SELECT 15

File sdroot;
int countRoot  = 0;
int emptyFiles = 0;
long sizeRoot  = 0;

void sdCardInit() {
  Serial.print(F("Modul SD...\n"));
  
  Serial.print(F(" SD init... "));
  if (SD.begin(SD_CHIP_SELECT)) {
    Serial.print(F("OK\n"));
    
    if (SD.exists("index.htm")) {
      Serial.print(F(" index.htm... found\n"));
    } else {
        Serial.print(F(" Can't find index.htm\n"));
        return;      
      }
  } else {
      Serial.print(F("failed\n"));
      return;
    }
 
  sdroot = SD.open("/");
  checkDir(sdroot);
  Serial.print(F(" Files: ")); Serial.println(countRoot);
  Serial.print(F(" Size:  ")); Serial.println(sizeRoot);
  if (emptyFiles) {
    Serial.print(F(" Empty: ")); Serial.println(emptyFiles);
  }
  
  modulSd = MODUL_ENABLE;
  initDone_();
} // sdCardInit()


void checkDir(File dir) {
  countRoot  = 0;
  sizeRoot   = 0;
  emptyFiles = 0;
  Serial.print(F(" Check"));
  dir.rewindDirectory();
  while (true) {
    File entry = dir.openNextFile();
    if (!entry) {break;}
    countRoot++;
    if (countRoot %  10 == 0) {Serial.print(F("."));}
    if (countRoot % 400 == 0) {Serial.print(F("\n"));}
    if (entry.size() == 0) {emptyFiles++;}
    sizeRoot += entry.size();
    entry.close();
  }
 Serial.print(F("\n"));
}

#ifdef SD_INFO_FEATURE
  Sd2Card  card;
  SdVolume volume;
  SdFile   root;

  void sdInfoInit() {
    Serial.print(F("Modul SD Info... \n"));
    if (card.init(SPI_FULL_SPEED, SD_CHIP_SELECT)) {
      printSdType();
      printFsType();
      printSdSize();
      //printSdRoot();
      initDone_();
    } else {
        Serial.print(F("failed\n"));
      }
  }

  // SD info
  
  byte sdType() {
    switch(card.type()) {
      case SD_CARD_TYPE_SD1:  return 1; break;
      case SD_CARD_TYPE_SD2:  return 2; break;
      case SD_CARD_TYPE_SDHC: return 3; break;
                     default: return 0;
    }
  }

  byte sdFileSystem() {
    if (volume.init(card)) {return volume.fatType();}
                      else {return 0;}
  }

  int sdVolumeSize() {
    int k = 1000; // 1000 or 1024
    if (volume.init(card)) {
      double volumesize;
      volumesize = volume.blocksPerCluster();
      volumesize *= volume.clusterCount();
      volumesize *= 512;      // size in bytes
      volumesize /= k;        // size in Kbytes
      return volumesize /= k; // size in Mbytes
    } else {
        return 0;
      }
  }

  // print SD info

  boolean sdIsInit() {
    // try to open the partition FAT16 or FAT32
    if (volume.init(card)) {
      return true;
    } else {
        Serial.print(F("Not find partition\n"));
        return false;
      }
  }
  
  void printSdType() {
    Serial.print(F(" SD type:    "));
    switch(card.type()) {
      case SD_CARD_TYPE_SD1:  Serial.print(F("SD1")); break;
      case SD_CARD_TYPE_SD2:  Serial.print(F("SD2")); break;
      case SD_CARD_TYPE_SDHC: Serial.print(F("SDHC"));break;
                     default: Serial.print(F("Unknown"));
    }
    Serial.print(F("\n"));
  }

  void printFsType() {
    // type of the first FAT-type volume
    if (sdIsInit()) {
      Serial.print(F(" Filesystem: FAT"));
      Serial.println(volume.fatType(), DEC);
    }
  }
  
  void printSdSize() {
    int k = 1000; // 1000 or 1024
    // try to open the partition FAT16 or FAT32
    if (sdIsInit()) {
      double volumesize;
      volumesize = volume.blocksPerCluster();
      volumesize *= volume.clusterCount();
      volumesize *= 512; // SD card blocks are always 512 bytes

      Serial.print(F(" Cluster:    "));
      Serial.println(volume.blocksPerCluster() * 512);
      
      //Serialprint("Volume size (B):  ");
      //Serial.println(volumesize);
      
      volumesize /= k;
      //Serialprint("Volume size (KB): ");
      //Serial.println(volumesize);
      
      volumesize /= k;
      //Serialprint("Volume size (MB): ");
      //Serial.println(volumesize);

      int intSize = volumesize;
      
      Serial.print(F(" SD size:    "));
      Serial.print(intSize); Serial.print(F(" MB\n"));
    } // if (sdIsInit())
  } // printSdSize()

  void printSdRoot() {
    // try to open the partition FAT16 or FAT32
    if (sdIsInit()) {
      Serial.print(F("Found files:\n"));
      root.openRoot(volume);
      root.ls(LS_R | LS_DATE | LS_SIZE);
    }
  }
#endif // SD_INFO_FEATURE

#ifdef SD_FILES_FEATURE
  void sdFilesInit() {
    started("SD Files");
  }

  void printSdContent(File dir, int numTabs) {
     while(true) {
       File entry =  dir.openNextFile();
       if (! entry) {
         dir.rewindDirectory();
         break;
       } // no more files
       for (uint8_t i = 0; i < numTabs; i++) {
         Serial.print('\t');
       }
       Serial.print(entry.name());
       if (entry.isDirectory()) {
         Serialprint("/\n");
         printSdContent(entry, numTabs + 1);
       } else {
           Serial.print("\t");
           Serial.println(entry.size(), DEC);
         }
       entry.close();
     }
  }  // printSdContent( )

  void printDirectory(File dir) {
     while(true) {
       File entry = dir.openNextFile();
       if (!entry) {break;} // no more files
       if (!entry.isDirectory()) {
         Serial.println(entry.name());
       } 
       entry.close();
     }
  }

  String strDirectory(File dir) {
    String str = "";
    File entry;
    dir.rewindDirectory();
    while(true) {
      entry = dir.openNextFile();
      if (! entry) { // no more files
        dir.rewindDirectory();
        return str;
        break;
      }
      if (! entry.isDirectory()) {
        str += entry.name();
        str += " ";
      } 
      entry.close();
    }
  }
#endif // SD_FILES_FEATURE
